<?php

/**
 * @file
 * Handles exporting of terms.
 */

/**
 * Build an array of operations for exporting each term.
 */
function mover_export_terms_operations($callback) {
  $result = db_query("SELECT tid FROM {term_data} ORDER BY tid");
  while ($t = db_fetch_object($result)) {
    $operations[] = array($callback, array($t->tid));
  }
  $operations[] = array('mover_export_make_tarball_process', array('terms'));
  return $operations;
}

/**
 * Batch processing callback for exporting each term.
 */
function mover_export_terms_process($tid, &$context) {
  $term = taxonomy_get_term($tid);
  if ($term->tid) {
    $directory = variable_get('mover_export_directory', '');
    $data = mover_export_term($term);

    // Write the exported file
    $directory = variable_get('mover_export_directory', '');
    $subdirectory = mover_get_directory_for_file($directory, 'terms', $context);
    $filepath = $subdirectory .'/'. $term->tid .'.txt';
    file_put_contents($filepath, $data);

    $context['message'] = check_plain($term->name);
    $context['results'][] = check_plain($term->name);
  }
  else {
    $message = t('Unable to load term @tid', array('@tid' => $tid));
    drupal_set_message($message);
    $context['message'] = check_plain($message);
    $context['results'][] = $tid;
  }
}

/**
 *  Process a term object to plain text code.
 *  Set $format to some string if encoding should be handled by some module that
 *  will recognise the string.
 */
function mover_export_term($original_term) {
  if (isset($original_term->tid)) {
    $term = drupal_clone($original_term);

    // Term "path" should probably be renamed. It's really the term name,
    // as a hierarchy (i.e., the path to the term like About > Staff, or
    // Products & Services > Products > Widgets. By exporting the full
    // hierarchy, each term can be separately imported without any complex
    // mapping to other terms.
    $term_path = array();
    $parents = array_reverse(taxonomy_get_parents_all($term->tid));
    foreach ($parents as $parent) {
      $term_path[] = $parent->name;
    }
    $term->path = $term_path;

    // Add the vocabulary name to terms
    $term->vocabulary = db_result(db_query("SELECT name FROM {vocabulary} WHERE vid=%d", $term->vid));

    // Normally there is only one URL alias, but Drupal allows multiple
    // aliases, so to protect against that edge case we'll export all
    // URL aliases.
    $term->aliases = array();
    $result = db_query("SELECT dst FROM {url_alias} WHERE src='%s'", 'taxonomy/term/'. $term->tid);
    while ($alias = db_fetch_object($result)) {
      $term->aliases[] = $alias->dst;
    }

    // Export any URL redirects to this term (including redirects that point to the URL aliases)
    $term->redirects = array();
    $aliases = $term->aliases;
    $aliases[] = 'taxonomy/term/'. $term->tid;
    $result = db_query("SELECT source FROM {path_redirect}
      WHERE redirect IN (". db_placeholders($aliases, 'varchar') .")", $aliases);
    while ($redirect = db_fetch_object($result)) {
      $term->redirects[] = $redirect->source;
    }

    // Export taxonomy node relationships
    if (db_table_exists('taxonomynode')) {
      $term->taxonomynode = db_result(db_query("SELECT nid FROM {taxonomynode} WHERE tid=%d", $term->tid));
    }

    // Let other modules do special fixing up.
    // The function signature is: hook_mover_node_alter(&$node, $original_node, $method)
    // Where $method is 'export'
    drupal_alter('mover_export_term', $term, $original_term, 'export');
    $term = mover_remove_recursion($term);
    $code = mover_object_encode($term);
    return $code;
  }
}
